/**
 * @author Mark Larkin, Conway Institute, UCD. mark.larkin@ucd.ie
 *
 * Changes:
 *
 * 16-01-07,Nigel Brown(EMBL): Copied and modified from AlignmentWidget to
 * subclass from Q3ScrollView instead of QWidget. This class replaces
 * AlignmentWidget on Mac OS X to work around scrolling limitation in Qt4
 * QScrollArea. THIS CLASS SHOULD BE REMOVED WHEN QT4 QSCROLLAREA IS FIXED!
 *
 * 19-01-07,Nigel Brown(EMBL): Fixed text selection bug, whereby selection
 * past the first scroll pane was 'invisible', but appeared in the correct
 * location relative to the widget border, when scrolled back to the first
 * pane: mouseMoveEvent(), mousePressEvent(), mouseReleaseEvent() changed to
 * contentsMouseMoveEvent(), contentsMousePressEvent(),
 * contentsMouseReleaseEvent().
 *
 * 30-01-07,Nigel Brown(EMBL): Put back sizeHint().
 *
 * 20-02-07,Nigel Brown(EMBL): constructor and changeBoxSizeAndFontSize()
 * parameters reordered.
 *
 * 12-4-07, Mark Larkin, Removed destructor. No need to delete QObjects. 
 *
 * 25-04-07,Nigel Brown(EMBL): Removed keyPressEvent(), keyReleaseEvent(),
 * shiftPressed. Keyboard events handled by KeyController singleton. Fixed
 * problem in shift-selection of columns which did not extend selection
 * backwards by replacing with new procedure selectColumns().
 *
 * 04-05-07,Nigel Brown(EMBL): moved getFirstSelectedCol() and
 * getSecondSelectedCol() into AlignmentWidgetScroll.cpp.
 *
 * 27-06-07,Nigel Brown(EMBL): For screen layout balancing: renamed
 * numSequencesSoFar as numSequences, added numSequences2, which records the
 * number of sequences in the *other* profile window; Added
 * setNumSequences2(), signal numSequencesChanged().
 ****************************************************************************/

#ifndef ALIGNMENTWIDGETSCROLL_H
#define ALIGNMENTWIDGETSCROLL_H

#include <QFont>
#include <QPoint>
#include <QSize>
#include <QString>
//#include <QWidget>  //nige
#include <QVector>
#include <vector>
#include <string>
#include "clustalW/alignment/Alignment.h"
#include "ColorParameters.h"
//-#include <QKeyEvent>
#include <QPixmap>
#include <Q3ScrollView> //nige
class QMouseEvent;
class QPaintEvent;
class QMessageBox;
//-class QKeyEvent;

/**
 * The AlignmentWidgetScroll is used to display a multiple alignment. It
 * takes sequences in the form of QString objects. The conservation
 * information is displayed, if it is available.
 */
class AlignmentWidgetScroll : public Q3ScrollView
{
    Q_OBJECT

public:
    AlignmentWidgetScroll(int boxWidth, int boxSize, QFont* font); //nige
    void setHorizontalScrollBarPolicy(Qt::ScrollBarPolicy policy); //nige
    void setVerticalScrollBarPolicy(Qt::ScrollBarPolicy policy); //nige
    QSize sizeHint() const; //nige
    void changeBoxSizeAndFontSize(int _boxWidth, int _boxSize, QFont* fontPtr);
    void updateSizeHint();
    void setAlignment(const clustalw::SeqArray* align, 
                      clustalw::Alignment* _alignObjPtr);
    void updateDisplay(int firstSeq, int lastSeq, clustalw::Array2D<int>* exRes, int nhead,
                       QString secStruct);
    void updateLowScoreSeg();
    void showExceptionalResidues(bool show);
    bool getShowExceptionalResidues(){return showExceptionalRes;}
    void showLowScoreSeg(bool show, clustalw::Array2D<int>* _lowScoreRes);
    bool getShowLowScoreSeg(){return showLowScoreRes;}
    void setColorScheme(ColorParameters* colors);
    void setPrintoutColorScheme(ColorParameters colors);
    void setBlackAndWhite();
    void setBackGroundColoring(bool backgroundColor); 
    bool getBackGroundColor(){return backGroundColor;}
    void clearRangeSelection();
    QString getConservationInfo(){return QString(conservationInfo.c_str());}
    QString getSecStructInfo(){return secStructInfo;} 
    clustalw::Array2D<QColor> getPrintoutColorMask();
    ColorParameters* getPtrToCurrentColorScheme(){return currentColorScheme;}
    int getFirstSelectedCol();
    int getSecondSelectedCol();
    void setNumSequences2(int count); //nige

public slots:

signals:
    void characterSelected(const QString &character);
    void numSequencesChanged(int count); //nige

protected:
    void contentsMousePressEvent(QMouseEvent *event);
    void contentsMouseMoveEvent(QMouseEvent *event);
    void contentsMouseReleaseEvent(QMouseEvent *event);
    //void paintEvent(QPaintEvent *event); DISABLED,nige
    void selectColumns(int column); //nige
    void drawContents(QPainter *p, int cx, int cy, int cw, int ch); //nige
    QSize computeSize() const; //nige

private:
    char getAminoAcidCode(int i);
    void setUpWeakAndStrongGroups();
    void calcConserveInfo();
    void calculatePixMaps();
    void makeColorMask(clustalw::Array2D<QColor>* colMask, 
                        ColorParameters* colorScheme);
    void makeColorMask(clustalw::Array2D<int>* colMask, 
                        ColorParameters* colorScheme);                        
    void generatePixMaps();
    void generateSecStructurePixmaps();
    QString initConsensus();
    int getExtraRowNum() const;
    
    int firstSeq, lastSeq;
    QFont displayFont;
    int firstColumnSelected;
    int secondColumn;
    int boxSize;
    int boxWidth;
    int pixelsize;
    int alignLength;
    int numSequences;
    int numSequences2; //nige
    int offSetFromTop;
    int displayStartPos;

    QPixmap defaultPixMap;
    QPixmap defaultGreyPixMap;
    QString conserveInfo;

    QColor *selectedColor; // This will stay here!!!!! Others will go!!!
    
    QColor *lightGrey;
    QColor *slateGrey;
    QColor *backGroundGrey;

    bool conserveDisplay;
    bool mousePressed;
    const clustalw::SeqArray* alignment;
    clustalw::Alignment* alignObjPtr;
    bool alignmentSet;
    bool showAlignment;
    std::vector<std::string> strongGroup; 
    std::vector<std::string> weakGroup;
    std::string conservationInfo;    
    clustalw::Array2D<int>* exceptionalRes;
    bool exResLoaded;
    bool showExceptionalRes;
    std::string conservationChars;
    bool lowScoreResLoaded;
    bool showLowScoreRes;
    
    int colorScheme;
    
    clustalw::Array2D<int>* lowScoreRes;
    clustalw::Array2D<QPixmap*>* colorAlignment;
    clustalw::Array2D<QColor> colorMask;
    clustalw::Array2D<int> colMask;
    QColor defaultColor;
    clustalw::Array2D<QPixmap> coloredLetters;    
    std::vector<QPixmap>* selectedColoredRes;
    std::vector<QPixmap>* lowScoreColoredRes;
    std::vector<QPixmap>* exceptionalColoredRes;
    std::vector<QPixmap>* conservationPixmaps;
    std::vector<QPixmap>* secStructPixmaps;
    // Color scheme stuff!!!
    ColorParameters* currentColorScheme;
    ColorParameters printoutColorScheme;
    
    bool blackAndWhite;
    bool backGroundColor;
    int secStruct;
    QString secStructInfo;
    int defaultIndex;      
};

#endif //ALIGNMENTWIDGETSCROLL_H
