/**
 * @author Mark Larkin, Conway Institute, UCD. mark.larkin@ucd.ie
 * 
 * Changes:
 *
 * 16-01-07,Nigel Brown(EMBL): Copied and modified from HistogramWidget to
 * subclass from Q3ScrollView instead of QWidget. This class replaces
 * HistogramWidget on Mac OS X to work around scrolling limitation in Qt4
 * QScrollArea. THIS CLASS SHOULD BE REMOVED WHEN QT4 QSCROLLAREA IS FIXED!
 *
 * 22-01-07,Nigel Brown(EMBL): replaced sizeHint() since default widget height
 * was too high.
 *
 * 30-01-07,Nigel Brown(EMBL): Added setFrameStyle(QFrame::NoFrame) to
 * constructor. Added missing updateContents() to addHistogramInfo().
 *
 * 31-01-07,Nigel Brown(EMBL): changed && to || in updateBoxSize(). Changed
 * drawContents() to paint histogram 'bars' before writing ruler text as this
 * was effectively clipped by the histogram bar on the next pass. Made ruler
 * fontSize 9 instead of 7 for readability on Mac and adjusted ruler depth in
 * calculateHistogramBars().
 *
 * 06-02-07,Nigel Brown(EMBL): darker histogram bars for Mac.
 *
 * 20-02-07,Nigel Brown(EMBL): constructor and updateBoxSize() parameters
 * reordered.
 *
 * 12-4-07, Mark Larkin : Changed destructor. We dont need to delete QObjects. 
 *
 * 16-04-07,Nigel Brown(EMBL): removed destructor; arrayOfHeights not
 * allocated here.
 ****************************************************************************/

#include "HistogramWidgetScroll.h"
#include <QtGui>
#include <QImage>
//#include <QPixmap>
#include <QPoint>
#include <QSize>
#include <QString>
//#include <QWidget>  //nige
#include <cmath>
#include "compile.h"
#include <iostream>
using namespace std;

/**
 * This is the constructor for a HistogramWidgetScroll.
 * @param histogramValues[] The array of histogram values to be displayed.
 * @param length The length of the alignment.
 * @param boxSize The width of the histogram bars.
 * @param *parent The parent of this widget. This parameter is optional.
 */
HistogramWidgetScroll::HistogramWidgetScroll(int _boxWidth, int _boxSize)  //nige
  : Q3ScrollView()  //nige
{
    /*
     * Set up the initial values.
     */
    alignLength = 0;
    arrayOfHeights = 0; 
    isSet = false;
    boxSize = _boxSize;
    boxWidth = _boxWidth;
    heightOfBox = 1.0;
    //histogramImage = 0;
    
    displayStartPos = 0; 
    extraWhiteSpace = 5;
    lengthSmallBar = 1;
    lengthLargeBar = 3;
    setBackgroundRole(QPalette::Base);
    
    /*
     * Create a color to display the ruler in.
     */
    rulerColor = new QColor;
    rulerColor->setRgb(192,215,233);

    offSetFromTop = 10;

    heightAllowed = 50;
    
    setFrameStyle(QFrame::NoFrame);

    updateSizeHint();
}

//nige
void HistogramWidgetScroll::setHorizontalScrollBarPolicy(Qt::ScrollBarPolicy policy)
{
    if (Qt::ScrollBarAsNeeded == policy)
        setHScrollBarMode(Q3ScrollView::Auto);
    else if (Qt::ScrollBarAlwaysOff == policy)
        setHScrollBarMode(Q3ScrollView::AlwaysOff);
    else if (Qt::ScrollBarAlwaysOn == policy)
        setHScrollBarMode(Q3ScrollView::AlwaysOn);
}

//nige
void HistogramWidgetScroll::setVerticalScrollBarPolicy(Qt::ScrollBarPolicy policy)
{
    if (Qt::ScrollBarAsNeeded == policy)
        setVScrollBarMode(Q3ScrollView::Auto);
    else if (Qt::ScrollBarAlwaysOff == policy)
        setVScrollBarMode(Q3ScrollView::AlwaysOff);
    else if (Qt::ScrollBarAlwaysOn == policy)
        setVScrollBarMode(Q3ScrollView::AlwaysOn);
}

/**
 * This function overrides the parents sizeHint function. It gives an idea of 
 * the size this widget needs. 
 */
QSize HistogramWidgetScroll::sizeHint() const
{
    int width = alignLength * boxWidth;  //nige
    int height = (heightAllowed * heightOfBox) + extraWhiteSpace;  //nige
    
    //(void) printf("HistogramWidgetScroll::sizeHint(w=%d,h=%d)\n", width, height);

    return QSize(width, height);  //nige
}
/**/

/**
 * This function will be called every time the alignment may have changed size.
 */
void HistogramWidgetScroll::updateSizeHint()
{
    int width = alignLength * boxWidth;  //nige
    int height = (heightAllowed * heightOfBox) + extraWhiteSpace;  //nige
    
    //(void) printf("HistogramWidgetScroll::updateSizeHint(w=%d,h=%d)\n", width, height);

    resizeContents(width, height);  //nige
    //updateContents();  //nige
}

void HistogramWidgetScroll::addHistogramInfo(std::vector<int>* histogramValues)
{
    arrayOfHeights = histogramValues;
    alignLength = arrayOfHeights->size();
    
    // Check each element is <= 100
    for(int i = 0; i < alignLength; i++)
    {
        if((*arrayOfHeights)[i] >= 100)
        {
            (*arrayOfHeights)[i] = 99; // Make sure 99 is the maximum value.    
        }
    }

    isSet = true;
    updateSizeHint();    
    calculateHistogramBars();    
    updateContents();  //nige
}

void HistogramWidgetScroll::clearHistogram()
{
    isSet = false;
    arrayOfHeights = 0;
    alignLength = 0;
            
    updateSizeHint();    
}

/**
 * This function is an event handler. It is called automatically when the widget 
 * needs to be repainted.
 * It draws the histogram and the ruler from the appropriate start and end positions.
 * @param *event A paint event object.
 * DISABLED - causes a 'shadow' of the canvas to be drawn! - nige
void HistogramWidgetScroll::paintEvent(QPaintEvent *event)
{
    QPainter painter(this);
    QRect rect = event->rect();
    int cx = rect.x();
    int cy = rect.y();
    int cw = rect.width();
    int ch = rect.height();
    (void) printf("HistogramWidgetScroll::paintEvent(%d,%d, %d,%d)\n", cx,cy, cw,ch);
    drawContents(&painter, cx, cy, cw, ch);
}
*/

void HistogramWidgetScroll::drawContents(QPainter *painter, int cx, int cy, int cw, int ch)
{
    painter->fillRect(cx, cy, cw, ch, QBrush(Qt::white));
    painter->setBackgroundMode(Qt::OpaqueMode);
    //painter->setBackground(QBrush(Qt::white));
    
    //rulerFont.setPixelSize(7); //nige: 7 unreadable on Mac
    rulerFont.setPixelSize(9);
    painter->setFont(rulerFont);    
    
    QFontMetrics fontMetrics(rulerFont);    
    
    int offSetForRuler = fontMetrics.height() - 1;
       
    // Set up the begin and end columns
    int beginColumn = cx        / boxWidth;
    int endColumn   = (cx+cw-1) / boxWidth;
    
    int index = 0;
    int rulerNum;
    int xPos;
    int yPos = 0;        
    
    //(void) printf("HistogramWidgetScroll::drawContents(x=%d,y=%d, w=%d,h=%d)(bw=%d,bh=%d) -> [bc=%d,ec=%d]\n",
    //              cx,cy, cw,ch, boxWidth,boxSize, beginColumn,endColumn);

    if(isSet && arrayOfHeights)
    {
        //31-01-07,nige: the histogram vertical stripes must be painted first
        //otherwise the current stripe may paint over the last ruler text.
        for (int column = beginColumn; column <= endColumn; ++column) 
        {
            if(column >= 0 && column < arrayOfHeights->size())
            {
                index = (*arrayOfHeights)[column];
                if(index >= 0 && index < histogramBars.size() &&
                   !histogramBars[index].isNull())
                {
                    painter->drawPixmap((column) * boxWidth, 0, histogramBars[index]);
                }
            }
        }
        for (int column = beginColumn; column <= endColumn; ++column) 
        {
            if(column >= 0 && column < arrayOfHeights->size())
            {
                rulerNum = column + 1;
                if ((rulerNum % 10) == 0) 
                {   
                    painter->drawLine(column * boxWidth + (boxWidth / 2), offSetForRuler, 
                                      column * boxWidth + (boxWidth / 2), 
                                      offSetForRuler + lengthLargeBar);
                
                    xPos = column * boxWidth + (boxWidth / 2) - 
                            (fontMetrics.width(QString::number(rulerNum))) / 2;
                    
                    if(rulerNum >= 10000)
                    {
                        painter->drawText(xPos, yPos, 7 * boxWidth, boxSize,
                                          Qt::TextSingleLine, QString::number(rulerNum));
                    }
                    else if(rulerNum >= 1000 )
                    {
                        painter->drawText(xPos, yPos, 6 * boxWidth, boxSize,
                                          Qt::TextSingleLine, QString::number(rulerNum));
                    }
                    else if(rulerNum >= 100 )
                    {
                        painter->drawText(xPos, yPos, 5 * boxWidth, boxSize,
                                          Qt::TextSingleLine, QString::number(rulerNum));
                    }
                    else if(rulerNum >= 10 )
                    {
                        painter->drawText(xPos, yPos, 3 * boxWidth, boxSize,
                                          Qt::TextSingleLine, QString::number(rulerNum));
                    }
                    else
                    {
                        painter->drawText(xPos, yPos, boxWidth, boxSize,
                                          Qt::TextSingleLine, QString::number(rulerNum));
                    }
                }
                else
                {
                    if(rulerNum == 1)
                    {
                        painter->drawText((boxWidth / 2) - 
                                          (fontMetrics.width(QString::number(0))) / 2, yPos, boxWidth,
                                          boxSize, Qt::TextSingleLine, QString::number(rulerNum));
                    }
                    painter->drawLine(column * boxWidth + (boxWidth / 2), offSetForRuler, 
                                      column * boxWidth + (boxWidth / 2), 
                                      offSetForRuler + lengthSmallBar);
                }                
            }            
        }
    }
}

void HistogramWidgetScroll::updateBoxSize(int _boxWidth, int _boxSize)
{
    if (boxWidth != _boxWidth || boxSize != _boxSize)
    {
        boxWidth = _boxWidth;
        boxSize  = _boxSize;
        int width = alignLength * boxWidth;  //nige
        int height = (heightAllowed * heightOfBox) + extraWhiteSpace;  //nige

        //(void) printf("HistogramWidgetScroll::updateBoxSize(w=%d,h=%d)\n", width, height);

        resizeContents(width, height);  //nige
        calculateHistogramBars();
        updateContents();  //nige
    }
}

void HistogramWidgetScroll::calculateHistogramBars()
{
    histogramBars.clear();
    histogramBars.resize(100);
    int heightOfBar;
    for(int i = 0; i < histogramBars.size(); i++)
    {
        QPoint origin(0, height());
        histogramBars[i] = QPixmap(boxWidth, height());
        heightOfBar = (i * heightOfBox) / 3;
        QPainter painter(&histogramBars[i]);
        painter.fillRect(0, 0, boxWidth, height(), QBrush(Qt::white));
        painter.translate(origin);
#if CLU_OS_MAC
        painter.fillRect(0, 0, boxWidth, -heightOfBar, QBrush(Qt::gray));
#else
        painter.fillRect(0, 0, boxWidth, -heightOfBar, QBrush(Qt::lightGray));
#endif
        painter.fillRect(0, -height(), boxWidth, 14, QBrush(rulerColor->rgb())); //nige: was 11 with 7 font
    }
}

/* TEST
void HistogramWidgetScroll::contentsMousePressEvent(QMouseEvent *event)  //nige
{
    if (event->button() == Qt::LeftButton)
    {
        (void) printf("HistogramWidgetScroll::contents(w=%d,h=%d)\n", width(), height());
        QSize size = sizeHint();
        int width = alignLength * boxWidth;  //nige
        int height = (heightAllowed * heightOfBox) + extraWhiteSpace;  //nige
        (void) printf("HistogramWidgetScroll::sizeHint(w=%d,h=%d) ?= (%d, %d) <- [ha=%d, hob=%1.3g, ews=%d] (called)\n",
                      size.width(), size.height(),
                      width, height,
                      heightAllowed, heightOfBox, extraWhiteSpace);
    }
    else
    {
        QWidget::mousePressEvent(event);
    }
}
TEST end */
